 /******************************************************************************
 *
 *  Novell Software Developer Kit
 *
 *  Copyright (C) 2002-2006 Novell, Inc. All Rights Reserved.
 *
 *  THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
 *  USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE LICENSE AGREEMENT
 *  ACCOMPANYING THE SOFTWARE DEVELOPER KIT (SDK) THAT CONTAINS THIS WORK.
 *  PURSUANT TO THE SDK LICENSE AGREEMENT, NOVELL HEREBY GRANTS TO DEVELOPER A
 *  ROYALTY-FREE, NON-EXCLUSIVE LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS
 *  PRODUCT. NOVELL GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
 *  DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF DEVELOPER'S
 *  PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO DEVELOPER OR DEVELOPER'S
 *  CUSTOMERS WITH RESPECT TO THIS CODE.
 *
 ******************************************************************************/

#ifndef _lynx_1_namapi_h_H
#define _lynx_1_namapi_h_H

#include <ldap.h>
#include <namconfig.h>

/******************************************************************************
 * bit values for [Entry Rights] of access control in Novell eDirecroty 
 *****************************************************************************/ 

/* Allows a trustee to discover objects in the Novell eDirecroty tree.
 */ 
#define NAM_LDAP_DS_ENTRY_BROWSE      0x00000001L

/* Allows a trustee to create child objects (new objects that are subordinate
 * to the object in the Novell eDirectory tree.)
 */
#define NAM_LDAP_DS_ENTRY_ADD         0x00000002L

/* Allows a trustee to delete an object. This right does not allow a trustee
 * to delete a container object that has subordinate objects.
 */
#define NAM_LDAP_DS_ENTRY_DELETE      0x00000004L

/* Allows a trustee to rename the object.
 */
#define NAM_LDAP_DS_ENTRY_RENAME      0x00000008L

/* Gives a trustee all rights to an object and its attributes.
 */
#define NAM_LDAP_DS_ENTRY_SUPERVISOR  0x00000010L

/* Allows a trustee to inherit the rights granted in the ACL
 * and exercise them on subordinate objects.
 */
#define NAM_LDAP_DS_ENTRY_INHERIT_CTL 0x00000040L


/******************************************************************************
 * bit values for [All Attributes Rights] and attribute rights of access 
 * control in Novell eDirecroty
 *****************************************************************************/
/* Allows a trustee to compare a value with an attribute's value. This allows 
 * the trustee to see if the attribute contains the value without having 
 * rights to see the value.
 */
#define NAM_LDAP_DS_ATTR_COMPARE      0x00000001L

/* Allows a trustee to read an attribute value. This right confers
 * the Compare right.
 */
#define NAM_LDAP_DS_ATTR_READ         0x00000002L

/* Allows a trustee to add, delete, or modify an attribute value. This right
 * also gives the trustee the Self (Add or Delete Self) right.
 */
#define NAM_LDAP_DS_ATTR_WRITE        0x00000004L

/* Allows a trustee to add or delete its name as an attribute value on those
 * attributes that take object names as their values.
 */
#define NAM_LDAP_DS_ATTR_SELF         0x00000008L

/* Gives a trustee all rights to the object's attributes.
 */
#define NAM_LDAP_DS_ATTR_SUPERVISOR   0x00000020L

/* Allows a trustee to inherit the rights granted in the ACL and exercise
 * these attribute rights on subordinate objects.
 */
#define NAM_LDAP_DS_ATTR_INHERIT_CTL  0x00000040L

/* This bit will be set if the trustee in the ACL is a dynamic group 
 * and its dynamic members should be considered for ACL rights 
 * calculation purposes. If this bit is reset, the trustee's static 
 * members alone will be considered for rights calculation purposes.
 */
#define NAM_LDAP_DS_DYNAMIC_ACL       0x20000000L

#define NAM_SUCCESS 0
#define NAM_BAD_PARAMETER 1
#define NAM_NO_ENTRY_FOUND 2
#define NAM_NO_MEMORY 3
#define NAM_SYSTEM_ERROR 4

//#define PAM_NAM_PATH "/lib/security/pam_nam.so"
#define PAM_NAM_PATH PAMPATH"/pam_nam.so"

//++
//	Name:   int namGetUserFDN(IN const char * userName, OUT char * userFDN, IN const char * password )
//
//	Description:
//      	This API will return FDN of user in * userFDN parameter.
//				
//	
//	Parameters in: 
//
//				userName is a pointer to user name (Linux, contextless)
//
//				password	is a pointer to user password			
//
//	Parameters out: 
//
//				userFDN is a pointer to the buffer at least 512 bytes long
//
//	Values returned:    
//
//     NAM_SUCCESS					If success
//
//			NAM_BAD_PARAMETER 	Bad parameter was passed to the function
//
//     NAM_NO_ENTRY_FOUND	Could not find FDN entry for the userName
//
//	Notes:
//
//
//--
int namGetUserFDN(const char * userName, char * userFDN, const char * password);


//++
//	Name:   int namGetUserFDNfromUID(IN int uid, OUT char * userFDN)
//
//	Description:
//      	This API will return FDN of user in userFDN parameter.
//				
//	
//	Parameters in: 
//
//				uid Posix uid for the Linux user
//
//	Parameters out: 
//
//				userFDN is a pointer to the buffer at least 512 bytes long
//
//	Values returned:    
//
//     NAM_SUCCESS					If success
//
//			NAM_BAD_PARAMETER 	Bad parameter was passed to the function
//
//     NAM_NO_ENTRY_FOUND	Could not find FDN entry for the userName
//
//	Notes:
//
//
//--
int namGetUserFDNfromUID(int uid, char * userFDN);

//++
//	Name:   namGetFDNforWS(OUT char * wsFDN)
//
//	Description:
//      	This API will return FDN of current Workstation/Server in wsFDN parameter.
//				
//	
//	Parameters in: 
//
//				None
//
//	Parameters out: 
//
//				wsFDN is a pointer to the buffer at least 512 bytes long
//
//	Values returned:    
//
//     NAM_SUCCESS					If success
//
//			NAM_BAD_PARAMETER 	Bad parameter was passed to the function
//
//     NAM_NO_ENTRY_FOUND	Could not find FDN entry for the userName
//
//	Notes:
//
//
//--
int namGetFDNforWS(char * wsFDN);

//++
//	Name:   int namGetEffectivePrivForWS(IN const char * userName, IN const char * pass, OUT int * privileges )
//
//	Description:
//      	This API will return bit mask of ACL for the user to the current Unix Workstation object
//				
//				
//	
//	Parameters in: 
//
//				userName is a pointer to user name (Linux, contextless)
//
//				password	is a pointer to user password			
//
//	Parameters out: 
//
//				privileges is a pointer to the integer which will hold the bitmask of ACL for 
//				the user to the current Unix Workstation object
//
//	Values returned:    
//
//     NAM_SUCCESS					If success
//
//			NAM_BAD_PARAMETER 	Bad parameter was passed to the function
//
//     NAM_NO_ENTRY_FOUND	Could not find FDN entry for the userName
//
//	Notes:
//
//
//--
int namGetEffectivePrivForWS(const char * userName, const char * pass, int * privileges);

//++
//	Name:   int namGetLDAPServerDNSOrIP(OUT char * ldapServerName)
//
//	Description:
//      	This API will return DNS name or IP address string found in nam.conf. 
//				
//	
//	Parameters in: 
//			none
//
//	Parameters out: 
//			DNS name or IP address in * ldapServerName    
//
//	Values returned:    
//     NAM_SUCCESS         If success
//
//     NAM_BAD_PARAMETER		Bad parameter was passed to the function
//
//     NAM_NO_ENTRY_FOUND	Could not find LDAP server name in the config file
//
//
//	Notes:
//
//
//--
int namGetLDAPServerDNSOrIP(char * ldapServerName);

//++
//	Name:   int namSetLDAPServerDNSOrIP(IN const char * ldapServerName)
//
//	Description:
//      	This API will set prefered server field found in nam.conf to DNS name or IP address string. 
//				
//	
//	Parameters in: 
//			ldapServerName		LDAP server IP address or DNS name
//
//	Parameters out: 
//												none
//
//	Values returned:    
//     NAM_SUCCESS         If success
//
//     NAM_BAD_PARAMETER		Bad parameter was passed to the function
//
//     NAM_NO_ENTRY_FOUND	Could not find LDAP server name in the config file
//
//
//	Notes:
//
//
//--
int namSetLDAPServerDNSOrIP(const char * ldapServerName);


//++
//	Name:   int namGetLDAPServerPort(OUT int * ldapServerPort)
//
//	Description:
//     This API will return port number found in nam.conf. 
//				
//	
//	Parameters in: none
//
//	Parameters out: 
//			ldapServerPort pointer to int which will have port number for LDAP
//			if SSL was specified in config file this port will be 636, otherwise 389
//
//	Values returned:    
//     NAM_SUCCESS         If success
//
//     NAM_BAD_PARAMETER		Bad parameter was passed to the function
//
//     NAM_NO_ENTRY_FOUND	Could not find LDAP server port in the config file
//
//
//	Notes:
//
//
//--
int namGetLDAPServerPort(int * ldapServerPort);


//++
//	Name:   int namGetLDAPServerCA(char * ldapServerCA, int * fileEncoding)
//
//	Description:
//     This API will return port number found in nam.conf. 
//				
//	
//	Parameters in: none
//
//	Parameters out: 
//			ldapServerCA 	- pointer to the buffer which will store CA location.
//			fileEncoding -	pointer to the int which will have encoding type.
//											LDAPSSL_CERT_FILETYPE_DER for DER
//											LDAPSSL_CERT_FILETYPE_B64 for B64 
//
//	Values returned:    
//     NAM_SUCCESS         If success
//
//     NAM_BAD_PARAMETER		Bad parameter was passed to the function
//
//     NAM_NO_ENTRY_FOUND	Could not find LDAP server port in the config file
//
//			NAM_NO_MEMORY 			Could not allocate memory
//
//	Notes:
//
//
//--
int namGetLDAPServerCA(char * ldapServerCA, int * fileEncoding);

//++
//	Name:   int namGetLDAPHandle(LDAP ** ldapHandle, const char * LDAPReplicaServer);
//
//	Description:
//     This API will return ldapHandle ready to do ldap_bind. 
//				
//	
//	Parameters in: 
//			LDAPReplicaServer - Optional parameter, which may contain LDAP server IP address or DNS
//													If this parameter is NULL API will use LDAP server IP/DNS from nam.conf
//													file only
//
//	Parameters out: 
//			ldapHandle - pointer to LDAP handle
//
//	Values returned:    
//     NAM_SUCCESS         If success
//
//     NAM_BAD_PARAMETER		Bad parameter was passed to the function
//
//     NAM_NO_ENTRY_FOUND	Could not find LDAP server port in the config file
//
//			NAM_NO_MEMORY 			Could not allocate memory
//
//	Notes:
//
//
//--
int namGetLDAPHandle(LDAP ** ldapHandle, const char * LDAPReplicaServer);

//++
//	Name:   int namGetAltLDAPServerList(OUT char * ldapServerList)
//
//	Description:
//      	This API will return comma separated list of DNS names or IP addresses 
//			for alternative LDAP servers found in nam.conf. 
//				
//	
//	Parameters in: 
//			none
//
//	Parameters out: 
//			ldapServerList		pointer to the buffer at least 2560 bytes long which will
//								contain the list 
//
//	Values returned:    
//     NAM_SUCCESS         	If success
//
//     NAM_BAD_PARAMETER	Bad parameter was passed to the function
//
//     NAM_NO_ENTRY_FOUND	Could not find list for alternative LDAP servers in the config file
//
//
//	Notes:
//
//
//--
int namGetAltLDAPServerList(char * ldapServerList);

//++
//	Name:   int namGetProxyUserAndPwd(OUT char * proxyUser, OUT char * proxyUserPwd)
//
//	Description:
//      	This API will return proxy User and proxy User Password
//				
//	
//	Parameters in: 
//			none
//
//	Parameters out: 
//			proxyUser			pointer to the buffer at least 512 bytes long which will
//										contain the proxy user name 
//			proxyUserPwd	pointer to the buffer at least 128 bytes long which will
//										contain the proxy user password 
//
//	Values returned:    
//     NAM_SUCCESS         	If success
//
//     NAM_BAD_PARAMETER	Bad parameter was passed to the function
//
//     NAM_NO_ENTRY_FOUND	Could not find list for alternative LDAP servers in the config file
//
//
//	Notes:
//
//
//--
int namGetProxyUserAndPwd(char * proxyUser, char * proxyUserPwd);

//++
//	Name:   int namGetUserFDNfromUIDCacheOnly(IN int uid, OUT char * userFDN)
//
//	Description:
//      	This API will return FDN of user in userFDN parameter.
//				
//	
//	Parameters in: 
//
//				uid Posix uid for the Linux user
//
//	Parameters out: 
//
//				userFDN is a pointer to the buffer at least 512 bytes long
//
//	Values returned:    
//
//     	NAM_SUCCESS			If success
//
//		NAM_BAD_PARAMETER 	Bad parameter was passed to the function
//
//     	NAM_NO_ENTRY_FOUND	Could not find FDN entry for the userName
//
//	Notes:
//
//
//--
int namGetUserFDNfromUIDCacheOnly(int uid, char * userFDN);


//
//	Function pointers provided for convinience of consumers for these APIs.
//
int (*pNamGetUserFDN)	(const char * userName, char * userFDN, const char * password);
int (*pNamGetLDAPServerDNSOrIP) (char * ldapServerName);
int (*pNamSetLDAPServerDNSOrIP) (const char * ldapServerName);
int (*pNamGetLDAPServerPort)	(int * ldapServerPort);
int (*pNamGetLDAPServerCA)	(char * ldapServerCA, int * fileEncoding);
int (*pNamGetLDAPHandle) (LDAP **ldapHandle, const char *LDAPReplicaServer);
int (*pNamGetUserFDNfromUID)	(int uid, char * userFDN);
int (*pNamGetFDNforWS) (char * wsFDN);
int (*pNamGetEffectivePrivForWS) (const char * userName, const char * pass, int * privileges);
int (*pNamGetAltLDAPServerList)(char * ldapServerList);
int (*pNamGetProxyUserAndPwd)(char * proxyUser, char * proxyUserPwd);
int (*pNamGetUserFDNfromUIDCacheOnly)	(int uid, char * userFDN);

#endif
